from __future__ import absolute_import
import re
import json

from ..models import FundingAccountToken
from ..exceptions import ApiException
from ..utils import LoggerUtil
from .base_request import BaseRequest
from .funding_account_api import FundingAccountApi
from ..api_client import ApiClient
from ..models import ConfirmAddFundAcctRequest
from ..models import ConfirmAddFundAcctResponse
from ..models import Error
from ..utils import TimeUtil
from ..utils import ValidationUtil
from ..utils.encryption_util import EncryptionUtil


class FundingAccount(BaseRequest, object):
    logger = LoggerUtil.setup_logger(__name__)
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id_account': 'str',
        'digi_sign': 'str',
        'custom_fields': 'dict',
        'funding_account_token': 'FundingAccountToken'
    }

    attribute_map = {
        'id_account': 'id_account',
        'digi_sign': 'digi_sign',
        'custom_fields': 'custom_fields',
        'funding_account_token': 'funding_account_token'
    }

    def __init__(self, id_account=None):
        self._id_account = None

        if id_account is not None:
            self.id_account = id_account

    """def __init__(self, digi_sign=None, custom_fields=None, funding_account_token=None):

        self._digi_sign = None
        self._custom_fields = None
        self._funding_account_token = None

        if digi_sign is not None:
            self._digi_sign = digi_sign
        if custom_fields is not None:
            self.custom_fields = custom_fields
        if funding_account_token is not None:
            self._funding_account_token = funding_account_token
"""

    @property
    def id_account(self):
        return self._id_account

    @id_account.setter
    def id_account(self, id_account):
        self._id_account = id_account

    @property
    def digi_sign(self):
        return self._digi_sign

    @digi_sign.setter
    def digi_sign(self, digi_sign):
        self._digi_sign = digi_sign

    @property
    def custom_fields(self):
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, custom_fields):
        self._custom_fields = custom_fields

    @property
    def funding_account_token(self):
        return self._funding_account_token

    @funding_account_token.setter
    def funding_account_token(self, funding_account_token):
        self._funding_account_token = funding_account_token

    def with_custom_fields(self, custom_fields):
        self._custom_fields = custom_fields
        return self

    def with_token(self, funding_account_token, digi_sign):
        if isinstance(funding_account_token, FundingAccountToken):
            self._funding_account_token = funding_account_token
        else:
            self._funding_account_token = FundingAccountToken(funding_account_token)

        self._digi_sign = digi_sign
        return self

    def for_client(self, client_id, signature_key):
        self.client_id = client_id
        self.signature_key = signature_key
        return self

    def for_partner(self, client_id, signature_key):
        self.client_id = client_id
        self.signature_key = signature_key
        return self

    def add(self):
        FundingAccount.logger.info('confirm_add_funding_account_request')
        try:
            if ValidationUtil.is_valid_add_funding_account_request(self):

                decrypted_token_id = EncryptionUtil.decrypt(self._funding_account_token.token)
                is_valid_sign = EncryptionUtil.verify(plain_text=decrypted_token_id, signature=self.digi_sign)

                if is_valid_sign:
                    self._funding_account_token.token = EncryptionUtil.encrypt(plain_text=decrypted_token_id)
                    self.digi_sign = EncryptionUtil.sign(plain_text=decrypted_token_id)
                else:
                    raise ApiException(
                        status=0,
                        reason="Invalid token_id or digi_sign"
                    )

                api_client = ApiClient(client_id=self.client_id, signature_key=self.signature_key)
                confirm_add_fund_acct_request = ConfirmAddFundAcctRequest(self._funding_account_token)
                funding_account_api = FundingAccountApi(api_client)
                start_time = TimeUtil.get_time_in_milli_seconds()

                confirm_add_funding_account_response = funding_account_api.confirm_add_funding_account(
                    confirm_add_fund_acct_request=confirm_add_fund_acct_request,
                    client_id=self.client_id,
                    digi_sign=self.digi_sign,
                    is_update_call=False,
                    id_account=None)
                end_time = TimeUtil.get_time_in_milli_seconds()
                FundingAccount.logger.info(
                    'Time taken for confirm_add_funding_account_request: {}'.format(end_time - start_time))
                return confirm_add_funding_account_response
            else:
                raise ApiException(
                    status=400,
                    reason='Invalid confirm_add_funding_account_request.'
                )
        except BaseException as e:
            try:

                FundingAccount.logger.error("exception occured: {}".format(e))
                error_python_object = json.loads(format(e))['errors']
                error_message = error_python_object[0]['message']
                error_code = error_python_object[0]['code']
                error_field = error_python_object[0]['field']
                add_account_response = ConfirmAddFundAcctResponse()
                error_on_add = Error(error_code,error_message,error_field)
                add_account_response.errors = error_on_add
                return add_account_response
            except BaseException as ee:
                raise ApiException(
                status=400,
                #reason= format(ee)
                reason=format(ee)
                )

    def update(self):
        FundingAccount.logger.info('confirm_update_funding_account_request')
        try:
            if ValidationUtil.is_valid_add_funding_account_request(self):

                decrypted_token_id = EncryptionUtil.decrypt(self._funding_account_token.token)
                is_valid_sign = EncryptionUtil.verify(plain_text=decrypted_token_id, signature=self.digi_sign)

                if is_valid_sign:
                    self._funding_account_token.token = EncryptionUtil.encrypt(plain_text=decrypted_token_id)
                    self.digi_sign = EncryptionUtil.sign(plain_text=decrypted_token_id)
                else:
                    raise ApiException(
                        status=0,
                        reason="Invalid token_id or digi_sign"
                    )

                api_client = ApiClient(client_id=self.client_id, signature_key=self.signature_key)
                confirm_add_fund_acct_request = ConfirmAddFundAcctRequest(self._funding_account_token)
                funding_account_api = FundingAccountApi(api_client)
                start_time = TimeUtil.get_time_in_milli_seconds()

                confirm_add_funding_account_response = funding_account_api.confirm_add_funding_account(
                    confirm_add_fund_acct_request=confirm_add_fund_acct_request,
                    client_id=self.client_id,
                    digi_sign=self.digi_sign,
                    is_update_call=True,
                    id_account=self.id_account)
                end_time = TimeUtil.get_time_in_milli_seconds()
                FundingAccount.logger.info(
                    'Time taken for confirm_update_funding_account_request: {}'.format(end_time - start_time))
                return confirm_add_funding_account_response
            else:
                raise ApiException(
                    status=400,
                    reason='Invalid confirm_update_funding_account_request.'
                )
        except BaseException as e:
            try:
                FundingAccount.logger.error("exception occured: {}".format(e))
                error_python_object = json.loads(format(e))['errors']
                error_message = error_python_object[0]['message']
                error_code = error_python_object[0]['code']
                error_field = error_python_object[0]['field']
                update_account_response = ConfirmAddFundAcctResponse()
                error_on_update = Error(error_code,error_message,error_field)
                update_account_response.errors = error_on_update
                return update_account_response
            except BaseException as ee:
                raise ApiException(
                status=400,
                #reason= format(ee)
                reason='Failed to confirm_payment_request. {}'.format(ee)
                )